@@ -1,3 +1,7 @@
+from decimal import Decimal
+from datetime import datetime, date
+from django.utils.functional import Promise
+
 from ua2.MongoLog.document import LogDocument
 
 
@@ -12,14 +16,24 @@ class Logger(object):
         self.doc_type = doc_type
     
     def log(self, level, key, descr, **kwargs):
+        for key, value in kwargs.iteritems():
+            if isinstance(value, (Decimal, datetime, date, Promise)):
+                kwargs[key] = unicode(value)
+            
         log_document = self.doc_type()
         log_document.level = level
         log_document.key = key
-        log_document.descr = descr.format(**kwargs)
 
         for key,item in kwargs.iteritems():
             setattr(log_document, key, item)
 
+        while True:
+            try:
+                log_document.descr = descr.format(**kwargs)
+                break
+            except KeyError, e:
+                kwargs[e.args[0]] = '<None>'
+
         log_document.save(safe=False, validate=False)
 
     @property
@@ -43,7 +57,8 @@ class Logger(object):
 
     def iter_log(self, wrap_cls, qs):
         wrapper = wrap_cls()
-        for item in qs:
+        
+        for item in qs.ensure_index('stamp'):
             wrapper.data = item._data
             yield wrapper
 
